@@ -25,7 +25,7 @@
 #     mean PHRED qual score above Q20 into another file).
 #    ---------------------------------------------------------
 #    import fastqlib as u
-#    
+#
 #    input  = u.FastQSource('/path/to/file.fastq')
 #    output = u.FastQOutput('/path/to/output.fastq')
 #
@@ -35,6 +35,7 @@
 #    ---------------------------------------------------------
 
 
+import io
 import os
 import sys
 import gzip
@@ -43,7 +44,7 @@ import textwrap
 
 from IlluminaUtils.utils.helperfunctions import big_number_pretty_print
 from IlluminaUtils.utils.helperfunctions import predict_file_length
-from IlluminaUtils.utils.helperfunctions import remove_spaces 
+from IlluminaUtils.utils.helperfunctions import remove_spaces
 
 
 class FastQLibError(Exception):
@@ -65,8 +66,6 @@ class FastQLibError(Exception):
         return '\n'.join(error_message)
 
 
-
-
 class FastQEntry:
     def __getattr__(self, key):
         # on demand quality processing.
@@ -80,8 +79,9 @@ class FastQEntry:
             self.process_Q_list()
 
         return getattr(self, '_'.join(['process', key]))()
-    
-    def __init__(self, (header_line, sequence_line, optional_line, qual_scores_line), trim_from = 0, trim_to = sys.maxint, raw = False, CASAVA_version = '1.8', pos = None):
+
+    def __init__(self, FASTQ_entry, trim_from = 0, trim_to = sys.maxsize, raw = False, CASAVA_version = '1.8', pos = None):
+        (header_line, sequence_line, optional_line, qual_scores_line) = FASTQ_entry
         self.is_valid = False
         self.raise_errors = True
 
@@ -90,12 +90,12 @@ class FastQEntry:
 
         if not header_line.startswith('@'):
             if self.raise_errors:
-                raise FastQLibError, 'FASTQ file seems to be corrupted. Header line does not start with a "@" character (pos: %s).' % (str(pos) if pos else 'unknown') 
+                raise FastQLibError('FASTQ file seems to be corrupted. Header line does not start with a "@" character (pos: %s).' % (str(pos) if pos else 'unknown'))
             return None
 
         if not optional_line.startswith('+'):
             if self.raise_errors:
-                raise FastQLibError, 'FASTQ file seems to be corrupted. Optional line does not start with a "+" character (pos: %s).' % (str(pos) if pos else 'unknown')
+                raise FastQLibError('FASTQ file seems to be corrupted. Optional line does not start with a "+" character (pos: %s).' % (str(pos) if pos else 'unknown'))
             return None
 
 
@@ -138,13 +138,13 @@ class FastQEntry:
             self.Q_list = [ord(q) - 33 for q in self.qual_scores]
         else:
             self.Q_list = [ord(q) - 64 for q in self.qual_scores]
- 
+
         return self.Q_list
-    
+
     def process_Q_min(self):
         self.Q_min = numpy.min(self.Q_list)
         return self.Q_min
-        
+
     def process_Q_mean(self):
         self.Q_mean = numpy.mean(self.Q_list)
         return self.Q_mean
@@ -153,7 +153,7 @@ class FastQEntry:
         self.Q_std  = numpy.std(self.Q_list)
         return self.Q_std
 
-    def trim(self, trim_from = 0, trim_to = sys.maxint):
+    def trim(self, trim_from = 0, trim_to = sys.maxsize):
         self.trim_to = trim_to
         self.trim_from = trim_from
         self.sequence    = self.sequence[trim_from:self.trim_to]
@@ -163,9 +163,9 @@ class FastQEntry:
 class FileOutput(object):
     def __init__(self, file_path, compressed = False):
         self.file_path = file_path
-        
+
         self.compressed = compressed
-        
+
         if self.compressed:
             self.file_pointer = gzip.open(file_path, 'w')
         else:
@@ -184,7 +184,7 @@ class FastQOutput(FileOutput):
         # mistakes. Nope; I didn't overwrite my millions of sequences.
         # Not at all.
         if os.path.exists(file_path):
-            raise FastQLibError, 'Output file exists: "%s"' % file_path
+            raise FastQLibError('Output file exists: "%s"' % file_path)
 
         super(FastQOutput, self).__init__(file_path, compressed)
 
@@ -201,7 +201,7 @@ class FastQSource:
 
     ---------------------------------------------------------
     import fastq as u
-    
+
     input  = u.FastQSource('/path/to/file.fastq')
 
     while input.next(trim_to = 75):
@@ -210,14 +210,16 @@ class FastQSource:
     """
     def __init__(self, file_path, compressed = False):
         if not os.path.exists(file_path):
-            raise FastQLibError, 'Missing input file: "%s"' % file_path
-        
+            raise FastQLibError('Missing input file: "%s"' % file_path)
+
         self.pos = 0
         self.forced_raw = False
 
         self.compressed = compressed
         if self.compressed:
-            self.file_pointer = gzip.open(file_path)
+            # wrap it with TextIOWrapper to prevent gzip.open to return byte\
+            # objects.
+            self.file_pointer = io.TextIOWrapper(gzip.open(file_path, 'r'))
             self.file_length = None
         else:
             self.file_pointer = open(file_path, 'rU')
@@ -227,7 +229,7 @@ class FastQSource:
         self.percent_read = None
         self.p_available = False
         self.percent_counter = 0
-        
+
         # FIXME: check for actual CASAVA version
         self.CASAVA_version = '1.8'
 
@@ -239,22 +241,22 @@ class FastQSource:
         self.reset()
 
 
-    def next(self, trim_from = 0, trim_to = sys.maxint, raw = False):
+    def next(self, trim_from = 0, trim_to = sys.maxsize, raw = False):
         if self.forced_raw and not raw:
-            raise FastQLibError, "Header lines in your FASTQ file does not seem to be the ones illumina-utils expects to see\
+            raise FastQLibError("Header lines in your FASTQ file does not seem to be the ones illumina-utils expects to see\
                                   in a FASTQ file generated by CASAVA 1.8. If you call this funciton with 'raw = True'\
                                   parameter, all should be fine. If you are accessing this function through a client, or in\
                                   other words if you have no idea what this message is telling you, try to re-run the program\
                                   with --ignore-deflines parameter. If that parameter is not available to you, then please send\
-                                  an e-mail to a.murat.eren@gmail.com."
+                                  an e-mail to a.murat.eren@gmail.com.")
         self.entry = FastQEntry([self.file_pointer.readline().strip() for _ in range(0, 4)], trim_from, trim_to, raw, CASAVA_version = self.CASAVA_version, pos = self.pos)
-       
+
         if not self.entry.is_valid:
             return False
 
         self.pos += 1
 
-        if self.pos == 1 or self.pos % 1000 == 0:
+        if self.pos == 1 or self.pos % self.percent_step == 0:
             self.p_available = True
             if self.file_length:
                 self.percent_read = self.pos * 100 / self.file_length
@@ -266,7 +268,7 @@ class FastQSource:
     def reset(self):
         self.pos = 0
         self.file_pointer.seek(0)
-    
+
     def close(self):
         self.file_pointer.close()
 
@@ -275,7 +277,7 @@ class FastQSource:
             sys.stderr.write('\r%s %.2d%% -- (num pairs processed: %s) %s' % (prefix, self.percent_read, big_number_pretty_print(self.pos), postfix))
         else:
             sys.stderr.write('\r%s (num pairs processed: %s) %s' % (prefix, big_number_pretty_print(self.pos), postfix))
-        
+
         sys.stderr.flush()
         self.p_available = False
 
